@@ -85,21 +85,48 @@ class GitRepository(object):
             raise GitRepositoryError(
                 "Failed to get repository state at '%s'" % self.path)
         self._bare = False if out.strip() != 'true' else True
-        self._git_dir = '' if self._bare else '.git'
 
-    def __init__(self, path):
-        self._path = os.path.abspath(path)
-        self._bare = False
+    def _get_git_dir(self):
+        out, dummy, ret = self._git_inout('rev-parse', ['--git-dir'],
+                                          capture_stderr=True)
+        if ret:
+            raise GitRepositoryError(
+                "Failed to determine repos git-dir at '%s'" % self.path)
+        git_dir = out.strip()
+        if os.path.isabs(git_dir):
+            self._git_dir = git_dir
+        else:
+            self._git_dir = os.path.abspath(os.path.join(self.path, git_dir))
+
+    def _check_repo(self, path, toplevel):
         try:
             out, dummy, ret = self._git_inout('rev-parse', ['--show-cdup'],
+                                              cwd=path,
                                               capture_stderr=True)
-            if ret or out.strip():
-                raise GitRepositoryError("No Git repository at '%s': '%s'" % (self.path, out))
+            cdup = out.strip()
+            if ret:
+                raise GitRepositoryError("No Git repository at '%s': '%s'" % (path, cdup))
+            if toplevel and cdup:
+                raise GitRepositoryError("Not the toplevel of a Git repository at '%s': '%s'" % (path, cdup))
+            ret = os.path.abspath(os.path.join(path, cdup or '.'))
         except GitRepositoryError:
             raise  # We already have a useful error message
         except:
-            raise GitRepositoryError("No Git repository at '%s'" % self.path)
+            raise GitRepositoryError("No Git repository at '%s'" % path)
+        return ret
+
+    def __init__(self, path, toplevel=True):
+        """
+        @param path: path to git repo (or subdir)
+        @type path: C{str}
+        @param toplevel: whether path points to the toplevel dir of
+            git repository
+        @type toplevel: C{bool}
+        """
+        self._bare = False
+        self._path = self._check_repo(path, toplevel)
         self._check_bare()
+        self._get_git_dir()
 
     @staticmethod
     def __build_env(extra_env):
@@ -120,7 +147,7 @@ class GitRepository(object):
         @type args: C{list}
         @param extra_env: extra environment variables to pass
         @type extra_env: C{dict}
-        @param cwd: directory to swith to when running the command, defaults to I{self.path}
+        @param cwd: directory to switch to when running the command, defaults to I{self.path}
         @type cwd: C{str}
         @return: stdout, return code
         @rtype: C{tuple} of C{list} of C{str} and C{int}
@@ -142,7 +169,7 @@ class GitRepository(object):
         return output, popen.returncode
 
     def _git_inout(self, command, args, input=None, extra_env=None, cwd=None,
-                   capture_stderr=False):
+                   capture_stderr=False, config_args=None):
         """
         Run a git command with input and return output
 
@@ -154,6 +181,8 @@ class GitRepository(object):
         @type args: C{list}
         @param extra_env: extra environment variables to pass
         @type extra_env: C{dict}
+        @param cwd: directory to switch to when running the command, defaults to I{self.path}
+        @type cwd: C{str}
         @param capture_stderr: whether to capture stderr
         @type capture_stderr: C{bool}
         @return: stdout, stderr, return code
@@ -161,14 +190,19 @@ class GitRepository(object):
         """
         if not cwd:
             cwd = self.path
-        return self.__git_inout(command, args, input, extra_env, cwd, capture_stderr)
+        return self.__git_inout(command, args, input, extra_env, cwd, capture_stderr, config_args)
 
     @classmethod
-    def __git_inout(cls, command, args, input, extra_env, cwd, capture_stderr):
+    def __git_inout(cls, command, args, input, extra_env, cwd, capture_stderr, config_args=None):
         """
         As _git_inout but can be used without an instance
         """
-        cmd = ['git', command] + args
+        config_opts = []
+        config_args = config_args or []
+        for arg in config_args:
+            config_opts.extend(['-c', arg])
+
+        cmd = ['git'] + config_opts + [command] + args
         env = cls.__build_env(extra_env)
         stderr_arg = subprocess.PIPE if capture_stderr else None
         stdin_arg = subprocess.PIPE if input is not None else None
@@ -1705,7 +1739,7 @@ class GitRepository(object):
         self._git_command("apply", args)
 
     def diff(self, obj1, obj2=None, paths=None, stat=False, summary=False,
-             text=False, ignore_submodules=True):
+             text=False, ignore_submodules=True, abbrev=None):
         """
         Diff two git repository objects
 
@@ -1727,6 +1761,7 @@ class GitRepository(object):
         @rtype: C{str}
         """
         options = GitArgs('-p', '--no-ext-diff')
+        config_args = GitArgs()
         if stat is True:
             options.add('--stat')
         elif stat:
@@ -1738,7 +1773,11 @@ class GitRepository(object):
         options.add_true(obj2, obj2)
         if paths:
             options.add('--', paths)
-        output, stderr, ret = self._git_inout('diff', options.args)
+        if abbrev is not None:
+            config_args.add('core.abbrev=%d' % abbrev)
+        output, stderr, ret = self._git_inout('diff',
+                                              options.args,
+                                              config_args=config_args.args)
         if ret:
             raise GitRepositoryError("Git diff failed")
         return output
@@ -1794,15 +1833,22 @@ class GitRepository(object):
         if ret:
             raise GitRepositoryError("Unable to archive %s: %s" % (treeish, err.strip()))
 
-    def collect_garbage(self, auto=False):
+    def collect_garbage(self, auto=False, prune=False, aggressive=False):
         """
         Cleanup unnecessary files and optimize the local repository
 
         param auto: only cleanup if required
         param auto: C{bool}
         """
-        args = ['--auto'] if auto else []
-        self._git_command("gc", args)
+        args = GitArgs('--quiet')
+        if prune is True:
+            args.add('--prune')
+        else:
+            args.add_true(prune, '--prune=%s' % prune)
+        args.add_true(aggressive, '--aggressive')
+        args.add_true(auto, '--auto')
+        self._git_command("gc", args.args)
+
 
 #{ Submodules
 
