@@ -7,14 +7,20 @@ import unittest
 
 import gbp.deb.git
 
+
 class DebianGitTestRepo(unittest.TestCase):
     """Scratch repo for a single unit test"""
 
-    def setUp(self):
+    def setUp(self, repo_cls=None):
+        name = 'test_repo'
         self.tmpdir = context.new_tmpdir(__name__)
 
-        repodir = self.tmpdir.join('test_repo')
-        self.repo = gbp.deb.git.DebianGitRepository.create(repodir)
+        if repo_cls is None:
+            repo_cls = gbp.deb.git.DebianGitRepository
+
+        repodir = self.tmpdir.join(name)
+        self.repodir = os.path.join(str(self.tmpdir), name)
+        self.repo = repo_cls.create(repodir)
 
     def tearDown(self):
         context.teardown()
@@ -36,6 +42,6 @@ class DebianGitTestRepo(unittest.TestCase):
             os.makedirs(d)
 
         with open(path, 'w+') as f:
-            content == None or f.write(content)
+            content is None or f.write(content)
         self.repo.add_files(name, force=True)
         self.repo.commit_files(path, msg or "added %s" % name)
